from StringIO import StringIO
from Products.CMFBibliographyAT.interfaces import IBibliographyExport
from Products.CMFCore.utils import getToolByName

from Products.CMFBibliographyAT.content.base import BaseEntry

# bibnotes_data should NOT be treated as a portal tool!
def afterInstall(self, reinstall=None, product=None):
    out = StringIO()
    out = justTheToolPlease(self, product, out)
    return out.getvalue()

def justTheToolPlease(self, qi_product, out):
    """Alters the Quickinstaller product information
    
    Limit the list of portalobjects to "bibnotes_tool";
    if the data file is included in the list, it gets
    deleted when BibNotes is uninstalled.
    """
    qi_product.portalobjects = ['bibnotes_tool']
    print >>out, ("""Limiting the QuickInstaller list of portal 
      objects to 'bibnotes_tool'; if the data file is included 
      in the list, it gets deleted when BibNotes is uninstalled.
      """,
      )
      
    return out

def install(self):
    out = StringIO()

    portal_types = getToolByName(self,'portal_types')
    portal_url = getToolByName(self,'portal_url')
    root = portal_url.getPortalObject()

    # Create top-level BibNotesFolder for citation data if it
    # does not already exist
    #if not 'bibnotes_data' in root.objectIds():
    #	new_id = root.invokeFactory(id='bibnotes_data'
    #	                          , type_name='BibNotesFolder')
    #	if new_id != 'bibnotes_data':
    #	    raise Exception
	    
    # Explicitly permit the creation of BibNotesFolder below Portal Site
    #types = portal_types.listTypeInfo()
    #for type in types:
    #	if type.getId() == 'Plone Site':
    #	    type.manage_changeProperties(allowed_content_types=('BibNotesFolder',))

    print >>out, 'Reconfirming security settings under bibnotes_data'
    bibnotes_data = getToolByName(self,'bibnotes_data')
    for username in bibnotes_data.keys():
	user_folder = bibnotes_data.restrictedTraverse(username)
	user_folder.manage_permission('Access contents information'
				    , ['Manager','Owner']
				    , acquire=1)
	user_folder.manage_permission('View'
				    , ['Manager','Owner']
				    , acquire=1)
    
    
    print >>out, 'Arranging to redirect views from notes to target objects'
    mytype = portal_types.getTypeInfo('BibNote')
    aliases = mytype.getMethodAliases()
    aliases['(Default)'] = 'bibnotes_redirect_view'
    aliases['view'] = 'bibnotes_redirect_view'
    mytype.setMethodAliases(aliases)
    
    # In the tuple, [3] is for metadata, [4] is for deletion
    print >>out, 'Creating new indexes as necessary:'
    new_indexes = []
    new_indexes.append(('bibnotesRefUID','p','f',1,1))
    new_indexes.append(('bibnotesTotalNotes','p','f',1,1))
    new_indexes.append(('bibnotesTotalRecommends','p','f',1,1))
    new_indexes.append(('bibnotesRecommend','p','f',0,1))
    new_indexes.append(('bibnotesToRead','p','f',0,1))
    new_indexes.append(('bibnotesBuzzDate','p','d',0,1))
    new_indexes.append(('bibnotesSomeReaders','p','k',1,1))
    # ... just in case ... okay, the last one *is* a bit silly ...
    new_indexes.append(('Authors','p','f',1,0))
    new_indexes.append(('Title','p','f',1,0))
    #
    # Reference Catalog
    #
    new_indexes.append(('bibnotesRecommend','r','f',1,1))
    new_indexes.append(('bibnotesToRead','r','f',1,1))
    new_indexes.append(('Creator','r','f',1,0))

    zcat={}
    zcat['p'] = 'portal_catalog'
    zcat['r'] = 'reference_catalog'

    fld={}
    fld['f'] = 'FieldIndex'
    fld['k'] = 'KeywordIndex'
    fld['d'] = 'DateIndex'
    
    for idx in new_indexes:
	out.write('  %s ... ' % (idx[0],))
	catalog = getToolByName(self,zcat[idx[1]])
	old_indexes = catalog.indexes()
	if not idx[0] in old_indexes:
	    catalog.manage_addIndex(idx[0],fld[idx[2]])
	    print >>out, 'created in %s as %s' % (zcat[idx[1]],fld[idx[2]])
	else:
	    print >>out, 'already exists in %s' % (zcat[idx[1]],)
    # offer metadata for bibnotesRefUID, for bibliography listings
    print >>out, 'Adding metadata columns:'
    for idx in new_indexes:
	catalog = getToolByName(self,zcat[idx[1]])
	if idx[3]:
	    try:
		catalog.addColumn(idx[0])
		print >>out, '  added %s' % (idx[0],)
	    except:
		pass

    for cat in ['reference_catalog','portal_catalog']:
	print >>out, 'Refreshing %s' % (zcat[idx[1]],)
	catalog = getToolByName(self,cat)
	catalog.refreshCatalog(clear=1)
    
    #hide annotations folder in the navigation (folder will
    # be created on the fly when pages are viewed)
    print >>out, 'Concealing annotations folder from portal navigation'
    portalProperties = getToolByName(self, 'portal_properties', None)
    if portalProperties is not None:
        navtreeProperties = getattr(portalProperties, 'navtree_properties', None)
        if navtreeProperties:
            navtreeProperties.idsNotToList = list(navtreeProperties.idsNotToList) + \
                                  [toolname for toolname in ['bibnotes_data','bibnotes_tool'] \
                                            if toolname not in navtreeProperties.idsNotToList]

    print >>out,'Attaching an "annotate" tab to the following types:'
    # Step through the bibliography types
    types = portal_types.listTypeInfo()
    portal_bibliography = getToolByName(self,'portal_bibliography')
    reference_types = portal_bibliography.getReferenceTypes()
    for type_obj in types:
	type_id = type_obj.getId()
	if type_id in reference_types:
	    out.write("    %s ... " % (type_obj.getId(),))
	    type_actions = type_obj.listActions()
	    if not 'annotate' in [x.getId() for x in type_actions]:
		type_obj.addAction(id='annotate',
		                   name='Annotate',
				   action='bibnotes_annotate',
				   condition=None,
				   permission='View',
				   category='object',
				   visible=1,
				   REQUEST=None)
		out.write("added")
	    else:
		out.write("already present, skipping")
	    out.write("\n")
    
    return out.getvalue()


def uninstall(self):
    out = StringIO()

    # Remove catalog entries
    new_indexes = []
    new_indexes.append(('bibnotesRefUID','p','f',1,1))
    new_indexes.append(('bibnotesTotalNotes','p','f',1,1))
    new_indexes.append(('bibnotesTotalRecommends','p','f',1,1))
    new_indexes.append(('bibnotesRecommend','p','f',0,1))
    new_indexes.append(('bibnotesToRead','p','f',0,1))
    new_indexes.append(('bibnotesBuzzDate','p','d',0,1))
    new_indexes.append(('bibnotesSomeReaders','p','k',1,1))
    # ... just in case ... okay, the last one *is* a bit silly ...
    new_indexes.append(('Authors','p','f',1,0))
    new_indexes.append(('Title','p','f',1,0))
    #
    # Reference Catalog
    #
    new_indexes.append(('bibnotesRecommend','r','f',1,1))
    new_indexes.append(('Creator','r','f',1,0))

    zcat={}
    zcat['p'] = 'portal_catalog'
    zcat['r'] = 'reference_catalog'

    print >>out, 'Removing indexes and metadata columns:'
    for idx in new_indexes:
	catalog = getToolByName(self,zcat[idx[1]])
	indexes = catalog.indexes()
	if idx[0] in indexes:
	    print >>out, '  %s (removed index)' % (idx[0],)
	    catalog.manage_delIndex(idx[0])

	if idx[4]:
	    # couldn't find a method for listing metadata field names
	    try:
		print >> out, '  %s (removed column)' % (idx[0],)
		catalog.delColumn(idx[0])
	    except:
		pass
    
    for cat in ['reference_catalog','portal_catalog']:
	print >>out, 'Refreshing %s' % (cat,)
	catalog = getToolByName(self,cat)
	catalog.refreshCatalog(clear=1)
    
    print >>out,'Removing the "annotate" tab, if any, from the following types:'
    portal_types = getToolByName(self,'portal_types')
    types = portal_types.listTypeInfo()
    portal_bibliography = getToolByName(self,'portal_bibliography')
    reference_types = portal_bibliography.getReferenceTypes()
    for type_obj in types:
	type_id = type_obj.getId()
	if type_id in reference_types:
	    out.write("    %s ... " % (type_obj.getId(),))
	    type_actions = type_obj.listActions()
	    if 'annotate' in [x.getId() for x in type_actions]:
		pos = [x.getId() for x in type_actions].index('annotate')
		type_obj.deleteActions(selections=(pos,))
		out.write("deleted")
	    else:
		out.write("none present, skipping")
	    out.write("\n")
    
    return out.getvalue()

