import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.CMFPlone.tests import PloneTestCase
from Products.BibNotes.tests import setup

from AccessControl.SecurityManagement import newSecurityManager, getSecurityManager

class TestNoteCreation(PloneTestCase.PloneTestCase):
    '''Test creation of an annotation'''

    ZopeTestCase._print('Will test note creation and retrieval\n')
    
    def afterSetUp(self):
        pass

    #
    # Most of these depend on what went before.  For unit tests,
    # build the nesting into utility functions, and test for
    # failure separately, in the test functions that call them.
    #
    
    def loginAsDefaultUser(self):
        DEFAULT_USER = PloneTestCase.default_user
	user = self.portal.acl_users.getUserById(DEFAULT_USER).__of__(self.portal.acl_users)
	newSecurityManager(None, user)
	return (user,user.getUserName())

    def getEmptyBibFolder(self):
	user, userid = self.loginAsDefaultUser()
        uf = self.portal.portal_membership.getHomeFolder(userid)
	if not 'bib_folder' in uf.objectIds():
	    uf.invokeFactory(type_name = "BibliographyFolder",
			     id = 'bib_folder')
        return (uf, 'bib_folder')

    def createArticle(self):
	uf, folderid = self.getEmptyBibFolder()
	bf = uf.restrictedTraverse(folderid)
        bf.invokeFactory(type_name = 'ArticleReference',
                         id = 'test_article')
	return (bf,'test_article')

    def getPopulatedBibFolder(self):
        uf, folderid = self.getEmptyBibFolder()
	bf = uf.restrictedTraverse(folderid)
	if not 'wagnild2002' in bf.objectIds():
	    bib_source = open(setup.MIXED_BAG_BIB, 'r').read()
	    bf.processImport(bib_source, 'mixed-bag.bib')
        return (bf,'wagnild2002')

    #
    # Not used by default; a chatty alternative to getPopulatedBibFolder
    # that spills the content of the citation DB to the monitor.
    #
    def verbosePopulatedBibFolder(self):
	bf = self.getPopulatedBibFolder()
	for item in bf.objectValues():
	    schema = item.Schema()
	    fields = schema.fields()
	    for field in fields:
		fieldname = field.getName()
		accessor = field.getAccessor(schema)
		ZopeTestCase._print('\n  %s: %s' % (fieldname,accessor()))
	return (bf, 'bib_folder')

    def getUserNotesDataFolder(self):
	user,userid = self.loginAsDefaultUser()
	#
	# This kludges around an error thrown up by the proxy-role
	# script that creates user directories in the note store.
	#
	self.portal.bibnotes_data.manage_permission('Modify portal content',['Manager','Member'],1)
	self.portal.bibnotes_open_folder()
	self.portal.bibnotes_data.manage_permission('Modify portal content',['Manager'],1)
	return (self.portal.bibnotes_data, userid)

    def openNoteObject(self):
	bd, userid = self.getUserNotesDataFolder()
	udata = bd.restrictedTraverse(userid)
	
	bf, entryid = self.getPopulatedBibFolder()
	target = bf.wagnild2002

	refuid = target.UID()
	self.portal.REQUEST.form['bibnotes_stub'] = '%s/%s' % (userid,refuid)
	target.bibnotes_open_content()
	newstub = self.portal.REQUEST.get('bibnotes_stub')
	target.REQUEST.form['bibnotes_stub'] = newstub
	note = self.portal.bibnotes_data.restrictedTraverse(newstub)
        return (target,note)

    def annotateOneEntry(self,textin):
	target, note = self.openNoteObject()

	target.REQUEST.form['annotation'] = textin
	target.bibnotes_edit()
	
	pcat = self.portal.portal_catalog
	res = pcat(meta_type='BibNote',bibnotesRefUID=target.UID())
	if res:
	    note = res[0].getObject()
	
	textout = note.Schema().getRawAnnotation()
	
	return (target,textout)
    
    def markOneEntryToRead(self):
	target, note = self.openNoteObject()

	target.REQUEST.form['bibnotesToRead'] = 1
	target.bibnotes_edit()
	
	pcat = self.portal.portal_catalog
	res = pcat(meta_type='BibNote',bibnotesRefUID=target.UID())
	if res:
	    note = res[0].getObject()
	
	return (target,note)
    
    def deleteEmptyNote(self):
	target, note = self.openNoteObject()

	target.REQUEST.form['bibnotesToRead'] = 1
	target.bibnotes_edit()
	
	rettext = ''
	
	pcat = self.portal.portal_catalog
	res = pcat(meta_type='BibNote',bibnotesRefUID=target.UID())
	if res:
	    note = res[0].getObject()
	else:
	    rettext = 'failed to create object before testing deletion'
	
	target, note = self.openNoteObject()
	target.REQUEST.form['bibnotesToRead'] = '0'
	target.REQUEST.form['annotation'] = None
	target.bibnotes_edit()
	
	res = pcat(meta_type='BibNote',bibnotesRefUID=target.UID())
	if res:
	    rettext = 'failed to delete note object'
	
	return (target,rettext)
    
    def clearToReadOnAnnotation(self,textin):
	target, note = self.openNoteObject()

	target.REQUEST.form['bibnotesToRead'] = 1
	target.bibnotes_edit()
	
	rettextwith = ''
	
	pcat = self.portal.portal_catalog
	res = pcat(meta_type='BibNote',bibnotesRefUID=target.UID())
	if res:
	    note = res[0].getObject()
	else:
	    rettextwith = 'failed to create object before testing deletion'
	
	target, note = self.openNoteObject()
	target.REQUEST.form['annotation'] = textin
	target.bibnotes_edit()
	
	reswith = pcat(meta_type='BibNote',
	               bibnotesRefUID=target.UID(),
		       bibnotesToRead=1)
	if reswith:
	    rettextwith = 'toggle failed to unset'
	
	reswithout = pcat(meta_type='BibNote',
  	                  bibnotesRefUID=target.UID(),
		          bibnotesToRead=0)

	if reswithout:
	    obj = reswithout[0].getObject()
	    rettextwithout = obj.Schema().getRawAnnotation()
	else:
	    rettextwithout = ''
			  
	return (rettextwith,rettextwithout)
    
    # the individual tests
    def testLoginAsDefaultUser(self):
	user,userid = self.loginAsDefaultUser()
	self.failUnless(userid == PloneTestCase.default_user)

    def testGetEmptyBibFolder(self):
	uf, folderid = self.getEmptyBibFolder()
	self.failUnless('bib_folder' in uf.objectIds())

    def testCreateArticle(self):
	bf, articleid = self.createArticle()
	self.failUnless('test_article' in bf.objectIds())
	
    def testGetPopulatedBibFolder(self):
	bf, entryid = self.getPopulatedBibFolder()
	self.failUnless(entryid in bf.objectIds())

    def testGetUserNotesDataFolder(self):
	bd, userid = self.getUserNotesDataFolder()
	self.failUnless(userid in bd.objectIds())

    def testOpenNoteObject(self):
	target,note = self.openNoteObject()
	self.failUnless(target.UID() == note.getId())
	
    def testAnnotateOneEntry(self):
	textin = 'Go Dragons!  Go Dragons !  Er, sorry, wrong entry there.  Perceptive and sensitive treatment of procedural issues in Japanese civil litigation.  There, that\'s better.'
	target, textout = self.annotateOneEntry(textin)
	self.failUnless(target.getId() == 'wagnild2002')
	self.failUnless(textout == textin,'\nIn: %s\nOut: %s\n' % (textin,textout))

    def testMarkOneEntryToRead(self):
	target, note = self.markOneEntryToRead()
	toread = note.Schema().getRawBibnotesToRead()
	self.failUnless(toread == 1)
    
    def testDeleteEmptyNote(self):
	target,rettext = self.deleteEmptyNote()
	self.failUnless(rettext == '',rettext)
	
    def testClearToReadOnAnnotation(self):
	textin = 'Another happy, happy annotation.  Work work work work!'
	rettextwith, rettextwithout = self.clearToReadOnAnnotation(textin)
	self.failUnless(rettextwith == '',rettextwith)
	self.failUnless(rettextwithout == textin,'\nIn: %s\nOut: %s\n' % (textin,rettextwithout))
	
    # end of the individual tests

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestNoteCreation))
    return suite

if __name__ == '__main__':
    framework()
