import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.CMFPlone.tests import PloneTestCase
from Products.BibNotes.tests import setup

from AccessControl.SecurityManagement import newSecurityManager, getSecurityManager

class TestLazyLists(PloneTestCase.PloneTestCase):
    '''Test index and lazy list retrieval'''

    ZopeTestCase._print('Will test index and lazy list retrieval\n')
    
    def afterSetUp(self):
        pass

    #
    # Most of these depend on what went before.  For unit tests,
    # build the nesting into utility functions, and test for
    # failure separately, in the test functions that call them.
    #
    
    def loginAsDefaultUser(self):
        DEFAULT_USER = PloneTestCase.default_user
	user = self.portal.acl_users.getUserById(DEFAULT_USER).__of__(self.portal.acl_users)
	newSecurityManager(None, user)
	return (user,user.getUserName())

    def getEmptyBibFolder(self):
	user, userid = self.loginAsDefaultUser()
        uf = self.portal.portal_membership.getHomeFolder(userid)
	if not 'bib_folder' in uf.objectIds():
	    uf.invokeFactory(type_name = "BibliographyFolder",
			     id = 'bib_folder')
        return (uf, 'bib_folder')

    def getPopulatedBibFolder(self):
        uf, folderid = self.getEmptyBibFolder()
	bf = uf.restrictedTraverse(folderid)
	if not 'wagnild2002' in bf.objectIds():
	    bib_source = open(setup.MIXED_BAG_BIB, 'r').read()
	    bf.processImport(bib_source, 'mixed-bag.bib')
        return (bf,'wagnild2002')

    def getUserNotesDataFolder(self):
	user,userid = self.loginAsDefaultUser()
	#
	# This kludges around an error thrown up by the proxy-role
	# script that creates user directories in the note store.
	#
	self.portal.bibnotes_data.manage_permission('Modify portal content',['Manager','Member'],1)
	self.portal.bibnotes_open_folder()
	self.portal.bibnotes_data.manage_permission('Modify portal content',['Manager'],1)
	return (self.portal.bibnotes_data, userid)

    def openNoteObject(self,id):
	bd, userid = self.getUserNotesDataFolder()
	udata = bd.restrictedTraverse(userid)
	
	bf, entryid = self.getPopulatedBibFolder()
	target = bf.restrictedTraverse(id)

	refuid = target.UID()
	self.portal.REQUEST.form['bibnotes_stub'] = '%s/%s' % (userid,refuid)
	target.bibnotes_open_content()
	newstub = self.portal.REQUEST.get('bibnotes_stub')
	target.REQUEST.form['bibnotes_stub'] = newstub
	note = self.portal.bibnotes_data.restrictedTraverse(newstub)
        return (target,note)

    def annotateOneEntry(self,id,textin,toread,recommend):
	target, note = self.openNoteObject(id)

	target.REQUEST.form['annotation'] = textin
	target.REQUEST.form['bibnotesToRead'] = str(toread)
	target.REQUEST.form['bibnotesRecommend'] = str(recommend)
	target.bibnotes_edit()
	
	pcat = self.portal.portal_catalog
	res = pcat(meta_type='BibNote',bibnotesRefUID=target.UID())
	if res:
	    note = res[0].getObject()
	
	textout = note.Schema().getRawAnnotation()
	
	return (target,textout)
    
    def setup(self):
	#
	# params[id][0] --> text for note
	# params[id][1] --> to-read toggle
	# params[id][2] --> recommend toggle
	#
	params = {}
	params['wagnild2002'] = ('',1,0)
	params['ikei2000'] = ('',1,0)
	params['sibbitt1997'] = ('',1,0)
	params['kindt1995'] = ('Note on kindt1995',0,0)
	params['stein2005'] = ('Note on stein2005',0,0)
	params['benasher1999'] = ('Note on benasher1999',0,0)
	params['chiu1980'] = ('Note on chiu1980',0,1)
	params['greenwood1995'] = ('Note on greenwood1995',0,1)
	params['phan2001'] = ('Note on phan2001',0,1)
    
	userid = getSecurityManager().getUser().getUserName()
	if not hasattr(self.portal.bibnotes_data,userid):
	    for p in params.keys():
		self.annotateOneEntry(p,params[p][0],params[p][1],params[p][2])
	    pcat = self.portal.portal_catalog
	    pcat.refreshCatalog()
	    get_transaction().commit()

    
    # the individual tests

    def testUnsortedFetchOfNotes(self):
	self.setup()
	pcat = self.portal.portal_catalog
	res = pcat(meta_type='BibNote')
	self.failUnless(res and len(res) == 9)
    
    def testUnsortedFetchOfBibliography(self):
	self.setup()
	pcat = self.portal.portal_catalog
	res = pcat(meta_type=['ArticleReference','BookReference'])
	self.failUnless(res and len(res) == 22)
    
    def testUnsortedFetchOfEntriesWIthNotes(self):
	self.setup()
	reference_types = self.portal.portal_bibliography.getReferenceTypes()
	userid = getSecurityManager().getUser().getUserName()
	udata = self.portal.bibnotes_data.restrictedTraverse(userid)
	userstash = [x.strip() for x in udata.objectIds()]
	pcat = self.portal.portal_catalog
	res = pcat(meta_type=reference_types,
		   bibnotesRefUID=userstash)
	self.failUnless(res and len(res) == 9,'result contains %d items' % (len(res),))
	
    def testAllEntriesSortedByDate(self):
	self.setup()
	reference_types = self.portal.portal_bibliography.getReferenceTypes()
	pcat = self.portal.portal_catalog
	res = pcat(meta_type=reference_types,
	           sort_on='publication_year',
		   sort_order='descending')
	res = [int(x.publication_year) for x in res]
	orig = []
	orig = orig + [2005,2004,2003,2002,2002,2002,2001,2001,2000]
	orig = orig + [1999,1999,1999,1997,1997,1996,1995,1995,1995]
	orig = orig + [1994,1991,1988,1980]
	self.failUnless(res == orig)
	
    def testToReadOnlySortedByDate(self):
	self.setup()
	userid = getSecurityManager().getUser().getUserName()
	pcat = self.portal.portal_catalog
	path = '/'.join(list(self.portal.bibnotes_data.getPhysicalPath()))
	res = pcat(meta_type='BibNote',
	           path='%s/%s' % (path,userid),
		   bibnotesToRead=1)
	self.failUnless(len(res) == 3)
	
    def testToReadOnlySortedByDate(self):
	self.setup()
	userid = getSecurityManager().getUser().getUserName()
	pcat = self.portal.portal_catalog
	path = '/'.join(list(self.portal.bibnotes_data.getPhysicalPath()))
	res = pcat(meta_type='BibNote',
	           path='%s/%s' % (path,userid),
		   bibnotesToRead=1,
		   sort_on='publication_year',
		   sort_order='descending')
	self.failUnless(len(res) == 3)
	rcat = self.portal.reference_catalog
	ids = [rcat.lookupObject(x.getId).getId() for x in res]
	self.failUnless(ids == ['wagnild2002','ikei2000','sibbitt1997'],'returned %s' % (ids,))
	
    def testRecommendOnlySortedByDate(self):
	self.setup()
	userid = getSecurityManager().getUser().getUserName()
	pcat = self.portal.portal_catalog
	path = '/'.join(list(self.portal.bibnotes_data.getPhysicalPath()))
	res = pcat(meta_type='BibNote',
	           path='%s/%s' % (path,userid),
		   bibnotesRecommend=1,
		   sort_on='publication_year',
		   sort_order='descending')
	self.failUnless(len(res) == 3)
	self.failUnless([x.publication_year for x in res] == ['2001','1995','1980'])

    def testMetadataFromNotes(self):
	self.setup()
	userid = getSecurityManager().getUser().getUserName()
	pcat = self.portal.portal_catalog
	path = '/'.join(list(self.portal.bibnotes_data.getPhysicalPath()))
	res = pcat(meta_type='BibNote',
		   SearchableText='stein')
        self.failUnless(res)
	self.failUnless(res[0].Authors == 'Stein, EZ')
	self.failUnless(res[0].publication_year == '2005')
	self.failUnless(res[0].Title == 'Note: Coming to America: Protecting Japanese Baseball Players Who Want to Play in the Major Leagues')

    # end of the individual tests

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestLazyLists))
    return suite

if __name__ == '__main__':
    framework()
